# cubic_bezier.py
# Representation and evaluation of cubic Bezier curves.

# Robert Bridson
# Released under the GPLv3

class CubicBezier(object):
    def __init__(self, finalT):
        object.__init__(self)
        # a CubicBezier contains two arrays: the knots of length n and
        # the values of length 3*(n-1)+1 (each interval has two interior
        # control points).
        self.knots = [0,finalT]
        self.values = [0, 0, 0, 0]

    def locate(self, t):
        """
        locate(t) returns (i,s): index of knot just below t, and 0<=s<=1
        """
        n = len(self.knots)
        if n<2:
            raise IndexError("locate needs at least two knots")
        m = len(self.values)
        if m != 3*(n-1)+1:
            raise ValueError("locate requires len(values)==3*(len(knots)-1)+1")
        if t <= self.knots[0]:
            return 0, 0
        if t >= self.knots[-1]:
            return n-1, 1
        i = 0
        while t>self.knots[i+1]:
            assert self.knots[i]<self.knots[i+1], "knots must be strictly increasing"
            i = i+1
        return i, (t-self.knots[i])/(self.knots[i+1]-self.knots[i])

    def eval(self, t):
        """
        eval(t) returns value of spline at t.
        """
        (i,s) = self.locate(t)
        # evaluate with de Casteljau and return the value
        # YOU FILL CODE IN BELOW, INSTEAD OF return 0
        return 0

    def findSplit(self, t):
        """
        findSplit(t) returns knot index, new knot, and new values for split at t

        That is, if you want to introduce a new knot at time t, findSplit
        gives you a triple of the index i of the existing knot just before t,
        the value of t itself, and a list of the five new control values that
        will go into the two new subintervals around t.

        Note this doesn't change the spline itself - see insertKnot for that.
        """
        (i,s) = self.locate(t)
        if s==0 or s==1:
            return None
        # YOU FILL CODE IN BELOW, INSTEAD OF [0, 0, 0, 0, 0],
        # REMEMBERING DE CASTELJAU'S METHOD.
        return i, t, [0, 0, 0, 0, 0]

    def combineIntervals(self, i):
        """
        combineIntervals(i) gives values to approximate spline on knots[i-1:i+1]

        This is for when you want to remove knot i, and need to come up with
        sensible new control point values --- the two new values are returned
        in a list.

        Note this doesn't change the spline itself - see removeKnot for that.
        """
        return [0, 0]

    def insertKnot(self, i, t, v):
        """
        insertKnot(i,t,v) inserts t after knots[i] and likewise v in values

        This actually changes the spline, inserting t and v.
        """
        self.knots.insert(i+1, t)
        self.values[3*i+1] = v[0]
        self.values[3*i+2] = v[1]
        self.values.insert(3*i+3, v[2])
        self.values.insert(3*i+4, v[3])
        self.values.insert(3*i+5, v[4])

    def removeKnot(self, i, v):
        """
        removeKnot(i,v) removes knot i and sets the two interior values to v

        This actually changes the spline.
        """
        self.knots.pop(i)
        self.values.pop(3*i+1)
        self.values.pop(3*i)
        self.values.pop(3*i-1)
        self.values[3*i-2] = v[0]
        self.values[3*i-1] = v[1]

