# framecontrol.py
# A PyQt4 widget for controlling frame number in animation software,
# scrubbing, playing, etc.
# We want to show the timeline with a slider, but also have text boxes for
# directly entering / seeing the current frame number, and also be able to
# enter / see the maximum frame number on the slider.

# Robert Bridson
# Released under the GPLv3

# Get the Python Qt bindings (PyQt4 or PySide)
from PyQt4.QtCore import *
from PyQt4.QtGui import *

class FrameControl(QWidget):
    """
    FrameControl lets you control the current frame and frame range.
    """

    # define the two signals FrameControl emits
    frameChanged = pyqtSignal(int)
    lastFrameChanged = pyqtSignal(int)
        
    def __init__(self, parent = None, maxFrame = 999):
        QWidget.__init__(self, parent)

        # the frame state (default)
        self.lastFrame = 99
        self.currentFrame = 0
        self.fps = 24

        # play timer (only active when playing)
        self.playTimer = QTimer(self)
        self.playTimer.timeout.connect(self._advancePlay)

        # the layout
        self.hbox = QHBoxLayout(self)

        # our slider
        self.slider = QSlider(Qt.Horizontal, self)
        self.slider.setTracking(True)
        self.slider.setTickPosition(QSlider.TicksBothSides)
        self.slider.setTickInterval(10)
        self.slider.setValue(0)
        self.hbox.addWidget(self.slider, 1)

        # current frame
        self.currentCounter = QSpinBox(self)
        self.currentCounter.setRange(0, self.lastFrame)
        self.currentCounter.setValue(self.currentFrame)
        self.hbox.addWidget(self.currentCounter, 0)

        # last frame
        self.lastCounter = QSpinBox(self)
        self.lastCounter.setRange(1, maxFrame)
        self.lastCounter.setValue(self.lastFrame)
        self.hbox.addWidget(self.lastCounter, 0)

        # play
        self.playStopButton = QPushButton("Play", self)
        self.hbox.addWidget(self.playStopButton, 0)

        # frame rate
        self.fpsBox = QDoubleSpinBox(self)
        self.fpsBox.setRange(0.1, 100)
        self.fpsBox.setValue(self.fps)
        self.hbox.addWidget(self.fpsBox, 0)

        # loop?
        self.loopBox = QCheckBox("Loop", self)
        self.hbox.addWidget(self.loopBox, 0)

        # connect up signals of components to sync methods
        self.slider.valueChanged.connect(self.setFrame)
        self.currentCounter.valueChanged.connect(self.setFrame)
        self.lastCounter.valueChanged.connect(self.setLastFrame)
        self.playStopButton.clicked.connect(self._clickPlayStop)
        self.fpsBox.valueChanged.connect(self._changeFrameRate)

    def setFrame(self, newFrame):
        newFrame = min(self.lastFrame, newFrame)
        self.currentFrame = newFrame
        self.slider.setValue(newFrame)
        self.currentCounter.setValue(newFrame)
        self.update()
        self.frameChanged.emit(newFrame)

    def setLastFrame(self, newLastFrame):
        if newLastFrame < 1:
            newLastFrame = 1
        if self.currentFrame>newLastFrame:
            self.setFrame(newLastFrame)
        self.lastFrame = newLastFrame
        self.slider.setMaximum(newLastFrame)
        self.currentCounter.setMaximum(newLastFrame)
        self.lastCounter.setValue(newLastFrame)
        self.update()
        self.lastFrameChanged.emit(newLastFrame)

    def _clickPlayStop(self):
        if self.playTimer.isActive(): # already running?
            self.playTimer.stop()
            self.playStopButton.setText("Play")
            self.playStopButton.update()
        else:
            self.playStopButton.setText("Stop")
            self.playStopButton.update()
            if self.currentFrame==self.lastFrame:
                self.setFrame(0) # go back to the beginning
            self.playTimer.start(1000/self.fps)

    def _advancePlay(self):
        newFrame = self.currentFrame + 1
        if newFrame>self.lastFrame:
            if self.loopBox.isChecked():
                self.setFrame(0)
            else:
                self.playTimer.stop()
                self.playStopButton.setText("Play")
                self.playStopButton.update()
        else:
            self.setFrame(newFrame)

    def _changeFrameRate(self, newFps):
        self.fps = newFps

